import React, { useContext, useEffect, useState } from 'react';
import ReactSwipe from 'react-swipe';
import { AppContext } from '../../layout/AppContext';
import { haveSameDay } from '../../utils/date';
import { CaloriesInput } from '../caloriesInput/CaloriesInput';
import { ItemActions } from './ItemActions';
import { ItemList } from './ItemList';
import { ListEditBox } from './ListEditBox';
import styles from './caloriesList.css';

function CaloriesList () {
  const {
    calorieItems = [],
    setCalorieItems,
    selectedDate,
    setSelectedDate,
    selectedItem,
    setSelectedItem
  } = useContext(AppContext);

  function addCalories (addedCalories: { title: string, count: number }) {
    const updatedCalories = [
      ...calorieItems,
      {
        ...addedCalories,
        timestamp: selectedDate
      }
    ];

    setCalorieItems(updatedCalories);
  }

  function removeItem () {
    setCalorieItems(calorieItems.filter((item) => item !== selectedItem));
    setSelectedItem();
  }

  function saveCalories (changedCalories: { title: string, count: number }) {
    if (!selectedItem) {
      return;
    }

    const itemIndex = calorieItems.indexOf(selectedItem);
    const updatedItem = {
      ...selectedItem,
      ...changedCalories
    };

    const updatedItems = [
      ...calorieItems.slice(0, itemIndex),
      updatedItem,
      ...calorieItems.slice(itemIndex + 1)
    ];

    setCalorieItems(updatedItems);
    setSelectedItem(updatedItem);
  }

  function onRowClick (item: CalorieValue) {
    if (selectedItem === item) {
      return setSelectedItem(undefined);
    }

    return setSelectedItem(item);
  }

  const swipeItem = (date: number, items: CalorieValue[]) => ({date, items});

  const dayCalories = [
    selectedDate - 3600 * 24 * 1000,
    selectedDate,
    selectedDate + 3600 * 24 * 1000
  ].map((date) => {
    return swipeItem(date, calorieItems.filter(({timestamp}) => haveSameDay(timestamp, date)));
  });

  return (
    <section className={styles.caloriesList}>
      <ReactSwipe swipeOptions={{
        continuous: false,
        speed: 250,
        startSlide: 1,
        transitionEnd (slide) {
          const newDate = slide === 0
            ? selectedDate - 3600 * 24 * 1000
            : slide === 2
              ? selectedDate + 3600 * 24 * 1000
              : selectedDate;

          setSelectedDate(newDate);
        }
      }} style={{
        child: {
          float: 'left',
          position: 'relative'
        },
        container: {
          flex: 1
        },
        wrapper: {
          height: '100%'
        }
      }}>
        {dayCalories.map(({date, items}) => {
          return (
            <ItemList key={date} {...{items, selectedItem, onRowClick}} />
          );
        })}
      </ReactSwipe>

      <div className={styles.editBoxes}>
        {selectedItem &&
        <ListEditBox>
          <ItemActions {...{removeItem}} />
        </ListEditBox>
        }

        <ListEditBox>
          <CaloriesInput {...{addCalories, saveCalories, item: selectedItem}} />
        </ListEditBox>
      </div>
    </section>
  );
}

export {
  CaloriesList
};
